# YML 2.0.36 language definition

# written by VB.

import re
from pyPEG import keyword, _and, _not

# pyPEG:
#
#   basestring:     terminal symbol (characters)
#   keyword:        terminal symbol (keyword)
#   matchobj:       terminal symbols (regex, use for scanning symbols)
#   function:       named non-terminal symbol, recursive definition
#                   if you don't want naming in output, precede name with an underscore
#   tuple:          production sequence
#   integer:        count in production sequence:
#                    0: following element is optional
#                   -1: following element can be omitted or repeated endless
#                   -2: following element is required and can be repeated endless
#   list:           options, choose one of them
#   _not:           next element in production sequence is matched only if this would not
#   _and:           next element in production sequence is matched only if this would, too

def comment(): return [re.compile(r"//.*"), re.compile("/\*.*?\*/", re.S)]
symbol = re.compile(r"[a-zA-Z_]\w*")
pointer = re.compile(r"\*[a-zA-Z_]\w*")
macro = re.compile(r"\%[a-zA-Z_]\w*")
reference = re.compile(r"\&[a-zA-Z_]\w*")
xmlSymbol = re.compile(r"[a-zA-Z][\-:a-zA-Z]*")
filename = re.compile(r"[^\s;]+")
literal = re.compile(r"""-?\d*\.\d*|-?\d+|".*?"|'.*?'""")
ws = re.compile(r"\s+")
def pyExp():        return "!", re.compile(r"(!=|\\!|[^!])+"), "!"
value = [literal, pyExp]

def tagQuote():     return re.compile(r"\].*|\<.*")
def lineQuote():    return re.compile(r"\|.*")
def quote():        return [re.compile(r"\d*>.*"), (literal, 0, ";")]
def parm():         return [([xmlSymbol, pyExp, pointer, macro], "=", [value, pointer]), value, pointer]
def _func():        return [symbol, pointer, reference], [("(", 0, parm, -1, (",", parm), ")"), (0, parm, -1, (",", parm))]
def pythonCall():   return keyword("python"), _func, ";"
def declParm():     return [xmlSymbol, pointer, macro], 0, ("=", literal)
def alias():        return keyword("alias"), xmlSymbol
def descend():      return re.compile(r"\+[a-zA-Z][\-:a-zA-Z]*")
def base():         return keyword("is"), symbol
def decl():         return symbol, 0, base, -1, descend, 0, ("(", 0, declParm, -1, (",", declParm), ")"), 0, alias, 0, content
def python():       return [re.compile(r"!!.*?!!", re.S), re.compile(r"!.*")]
def operator():     return keyword("define"), keyword("operator"), literal, keyword("as"), re.compile(r".*")
def in_ns():        return keyword("in"), xmlSymbol, [_decl, ("{", -2, _decl, "}")]
def _decl():        return keyword("decl"), decl, -1, (",", decl), ";"
def textsection():  return re.compile(r'(\|\|(\>*).*?\|\|(\>*))|(""".*?""")|(\>\>.*?\>\>)', re.S)
def _inner():       return [include, textsection, pythonCall, func, quote, lineQuote, tagQuote, pyExp]
def content():      return [_inner, ("{", -1, _inner, "}")]
def func():         return _not(keyword("decl")), _func, [";", (content, 0, ";")]
def reverse():      return keyword("reverse")
def ktext():        return keyword("text")
def include():      return keyword("include"), 0, reverse, 0, ktext, filename, 0, ";"
def ymlCStyle():    return -1, [_decl, in_ns, include, python, operator, tagQuote, func]
